mirror of https://lore.kernel.org/lkml/
 help / color / mirror / Atom feed
From: Steven Price <steven.price@arm.com>
To: kvm@vger.kernel.org, kvmarm@lists.linux.dev
Cc: Jean-Philippe Brucker <jean-philippe@linaro.org>,
	Catalin Marinas <catalin.marinas@arm.com>,
	Marc Zyngier <maz@kernel.org>, Will Deacon <will@kernel.org>,
	James Morse <james.morse@arm.com>,
	Oliver Upton <oliver.upton@linux.dev>,
	Suzuki K Poulose <suzuki.poulose@arm.com>,
	Zenghui Yu <yuzenghui@huawei.com>,
	linux-arm-kernel@lists.infradead.org,
	linux-kernel@vger.kernel.org, Joey Gouly <joey.gouly@arm.com>,
	Alexandru Elisei <alexandru.elisei@arm.com>,
	Christoffer Dall <christoffer.dall@arm.com>,
	Fuad Tabba <tabba@google.com>,
	linux-coco@lists.linux.dev,
	Ganapatrao Kulkarni <gankulkarni@os.amperecomputing.com>,
	Steven Price <steven.price@arm.com>
Subject: [PATCH v3 36/43] arm64: RME: Set breakpoint parameters through SET_ONE_REG
Date: Mon, 10 Jun 2024 14:41:55 +0100	[thread overview]
Message-ID: <20240610134202.54893-37-steven.price@arm.com> (raw)
In-Reply-To: <20240610134202.54893-1-steven.price@arm.com>

From: Jean-Philippe Brucker <jean-philippe@linaro.org>

Allow userspace to configure the number of breakpoints and watchpoints
of a Realm VM through KVM_SET_ONE_REG ID_AA64DFR0_EL1.

The KVM sys_reg handler checks the user value against the maximum value
given by RMM (arm64_check_features() gets it from the
read_sanitised_id_aa64dfr0_el1() reset handler).

Userspace discovers that it can write these fields by issuing a
KVM_ARM_GET_REG_WRITABLE_MASKS ioctl.

Signed-off-by: Jean-Philippe Brucker <jean-philippe@linaro.org>
Signed-off-by: Steven Price <steven.price@arm.com>
---
 arch/arm64/kvm/guest.c    |  2 ++
 arch/arm64/kvm/rme.c      |  3 +++
 arch/arm64/kvm/sys_regs.c | 21 ++++++++++++++-------
 3 files changed, 19 insertions(+), 7 deletions(-)

diff --git a/arch/arm64/kvm/guest.c b/arch/arm64/kvm/guest.c
index e17679c347ae..ad77e1343269 100644
--- a/arch/arm64/kvm/guest.c
+++ b/arch/arm64/kvm/guest.c
@@ -783,6 +783,7 @@ int kvm_arm_get_reg(struct kvm_vcpu *vcpu, const struct kvm_one_reg *reg)
 }
 
 #define KVM_REG_ARM_PMCR_EL0		ARM64_SYS_REG(3, 3, 9, 12, 0)
+#define KVM_REG_ARM_ID_AA64DFR0_EL1	ARM64_SYS_REG(3, 0, 0, 5, 0)
 
 /*
  * The RMI ABI only enables setting the lower GPRs (x0-x7) and PC.
@@ -805,6 +806,7 @@ static bool validate_realm_set_reg(struct kvm_vcpu *vcpu,
 	} else {
 		switch (reg->id) {
 		case KVM_REG_ARM_PMCR_EL0:
+		case KVM_REG_ARM_ID_AA64DFR0_EL1:
 			return true;
 		}
 	}
diff --git a/arch/arm64/kvm/rme.c b/arch/arm64/kvm/rme.c
index 66e124b4d25a..31e00c52551d 100644
--- a/arch/arm64/kvm/rme.c
+++ b/arch/arm64/kvm/rme.c
@@ -315,6 +315,7 @@ static int realm_create_rd(struct kvm *kvm)
 	void *rd = NULL;
 	phys_addr_t rd_phys, params_phys;
 	struct kvm_pgtable *pgt = kvm->arch.mmu.pgt;
+	u64 dfr0 = IDREG(kvm, SYS_ID_AA64DFR0_EL1);
 	int i, r;
 
 	if (WARN_ON(realm->rd) || WARN_ON(!realm->params))
@@ -346,6 +347,8 @@ static int realm_create_rd(struct kvm *kvm)
 	params->rtt_num_start = pgt->pgd_pages;
 	params->rtt_base = kvm->arch.mmu.pgd_phys;
 	params->vmid = realm->vmid;
+	params->num_bps = SYS_FIELD_GET(ID_AA64DFR0_EL1, BRPs, dfr0) + 1;
+	params->num_wps = SYS_FIELD_GET(ID_AA64DFR0_EL1, WRPs, dfr0) + 1;
 
 	if (kvm->arch.arm_pmu) {
 		params->pmu_num_ctrs = kvm->arch.pmcr_n;
diff --git a/arch/arm64/kvm/sys_regs.c b/arch/arm64/kvm/sys_regs.c
index 96d73b1867ad..7b05ecc369e0 100644
--- a/arch/arm64/kvm/sys_regs.c
+++ b/arch/arm64/kvm/sys_regs.c
@@ -1731,6 +1731,9 @@ static int set_id_aa64dfr0_el1(struct kvm_vcpu *vcpu,
 {
 	u8 debugver = SYS_FIELD_GET(ID_AA64DFR0_EL1, DebugVer, val);
 	u8 pmuver = SYS_FIELD_GET(ID_AA64DFR0_EL1, PMUVer, val);
+	u8 bps = SYS_FIELD_GET(ID_AA64DFR0_EL1, BRPs, val);
+	u8 wps = SYS_FIELD_GET(ID_AA64DFR0_EL1, WRPs, val);
+	u8 ctx_cmps = SYS_FIELD_GET(ID_AA64DFR0_EL1, CTX_CMPs, val);
 
 	/*
 	 * Prior to commit 3d0dba5764b9 ("KVM: arm64: PMU: Move the
@@ -1750,10 +1753,11 @@ static int set_id_aa64dfr0_el1(struct kvm_vcpu *vcpu,
 		val &= ~ID_AA64DFR0_EL1_PMUVer_MASK;
 
 	/*
-	 * ID_AA64DFR0_EL1.DebugVer is one of those awkward fields with a
-	 * nonzero minimum safe value.
+	 * ID_AA64DFR0_EL1.DebugVer, BRPs and WRPs all have to be greater than
+	 * zero. CTX_CMPs is never greater than BRPs.
 	 */
-	if (debugver < ID_AA64DFR0_EL1_DebugVer_IMP)
+	if (debugver < ID_AA64DFR0_EL1_DebugVer_IMP || !bps || !wps ||
+	    ctx_cmps > bps)
 		return -EINVAL;
 
 	return set_id_reg(vcpu, rd, val);
@@ -1836,10 +1840,11 @@ static int set_id_reg(struct kvm_vcpu *vcpu, const struct sys_reg_desc *rd,
 	mutex_lock(&vcpu->kvm->arch.config_lock);
 
 	/*
 	 * Once the VM has started the ID registers are immutable. Reject any
 	 * write that does not match the final register value.
 	 */
-	if (kvm_vm_has_ran_once(vcpu->kvm)) {
+	if (kvm_vm_has_ran_once(vcpu->kvm) || kvm_realm_is_created(vcpu->kvm)) {
 		if (val != read_id_reg(vcpu, rd))
 			ret = -EBUSY;
 		else
@@ -2321,7 +2326,9 @@ static const struct sys_reg_desc sys_reg_descs[] = {
 	  .set_user = set_id_aa64dfr0_el1,
 	  .reset = read_sanitised_id_aa64dfr0_el1,
 	  .val = ID_AA64DFR0_EL1_PMUVer_MASK |
-		 ID_AA64DFR0_EL1_DebugVer_MASK, },
+		 ID_AA64DFR0_EL1_DebugVer_MASK |
+		 ID_AA64DFR0_EL1_BRPs_MASK |
+		 ID_AA64DFR0_EL1_WRPs_MASK, },
 	ID_SANITISED(ID_AA64DFR1_EL1),
 	ID_UNALLOCATED(5,2),
 	ID_UNALLOCATED(5,3),
@@ -1836,10 +1840,11 @@ static int set_id_reg(struct kvm_vcpu *vcpu, const struct sys_reg_desc *rd,
 
 	/*
-	 * Once the VM has started the ID registers are immutable. Reject any
-	 * write that does not match the final register value.
+	 * Once the VM has started or the Realm descriptor is created, the ID
+	 * registers are immutable. Reject any write that does not match the
+	 * final register value.
 	 */
-- 
2.34.1


  parent reply	other threads:[~2024-06-10 13:44 UTC|newest]

Thread overview: 44+ messages / expand[flat|nested]  mbox.gz  Atom feed  top
2024-06-10 13:41 [PATCH v3 00/43] arm64: Support for Arm CCA in KVM Steven Price
2024-06-10 13:41 ` [PATCH v3 01/43] KVM: Prepare for handling only shared mappings in mmu_notifier events Steven Price
2024-06-10 13:41 ` [PATCH v3 02/43] kvm: arm64: pgtable: Track the number of pages in the entry level Steven Price
2024-06-10 13:41 ` [PATCH v3 03/43] kvm: arm64: Include kvm_emulate.h in kvm/arm_psci.h Steven Price
2024-06-10 13:41 ` [PATCH v3 04/43] arm64: RME: Handle Granule Protection Faults (GPFs) Steven Price
2024-06-10 13:41 ` [PATCH v3 05/43] arm64: RME: Add SMC definitions for calling the RMM Steven Price
2024-06-10 13:41 ` [PATCH v3 06/43] arm64: RME: Add wrappers for RMI calls Steven Price
2024-06-10 13:41 ` [PATCH v3 07/43] arm64: RME: Check for RME support at KVM init Steven Price
2024-06-10 13:41 ` [PATCH v3 08/43] arm64: RME: Define the user ABI Steven Price
2024-06-10 13:41 ` [PATCH v3 09/43] arm64: RME: ioctls to create and configure realms Steven Price
2024-06-10 13:41 ` [PATCH v3 10/43] kvm: arm64: Expose debug HW register numbers for Realm Steven Price
2024-06-10 13:41 ` [PATCH v3 11/43] arm64: kvm: Allow passing machine type in KVM creation Steven Price
2024-06-10 13:41 ` [PATCH v3 12/43] arm64: RME: Keep a spare page delegated to the RMM Steven Price
2024-06-10 13:41 ` [PATCH v3 13/43] arm64: RME: RTT tear down Steven Price
2024-06-10 13:41 ` [PATCH v3 14/43] arm64: RME: Allocate/free RECs to match vCPUs Steven Price
2024-06-10 13:41 ` [PATCH v3 15/43] arm64: RME: Support for the VGIC in realms Steven Price
2024-06-10 13:41 ` [PATCH v3 16/43] KVM: arm64: Support timers in realm RECs Steven Price
2024-06-10 13:41 ` [PATCH v3 17/43] arm64: RME: Allow VMM to set RIPAS Steven Price
2024-06-10 13:41 ` [PATCH v3 18/43] arm64: RME: Handle realm enter/exit Steven Price
2024-06-10 13:41 ` [PATCH v3 19/43] KVM: arm64: Handle realm MMIO emulation Steven Price
2024-06-10 13:41 ` [PATCH v3 20/43] arm64: RME: Allow populating initial contents Steven Price
2024-06-10 13:41 ` [PATCH v3 21/43] arm64: RME: Runtime faulting of memory Steven Price
2024-06-10 13:41 ` [PATCH v3 22/43] KVM: arm64: Handle realm VCPU load Steven Price
2024-06-10 13:41 ` [PATCH v3 23/43] KVM: arm64: Validate register access for a Realm VM Steven Price
2024-06-10 13:41 ` [PATCH v3 24/43] KVM: arm64: Handle Realm PSCI requests Steven Price
2024-06-10 13:41 ` [PATCH v3 25/43] KVM: arm64: WARN on injected undef exceptions Steven Price
2024-06-10 13:41 ` [PATCH v3 26/43] arm64: Don't expose stolen time for realm guests Steven Price
2024-06-10 13:41 ` [PATCH v3 27/43] arm64: rme: allow userspace to inject aborts Steven Price
2024-06-10 13:41 ` [PATCH v3 28/43] arm64: rme: support RSI_HOST_CALL Steven Price
2024-06-10 13:41 ` [PATCH v3 29/43] arm64: rme: Allow checking SVE on VM instance Steven Price
2024-06-10 13:41 ` [PATCH v3 30/43] arm64: RME: Always use 4k pages for realms Steven Price
2024-06-10 13:41 ` [PATCH v3 31/43] arm64: rme: Prevent Device mappings for Realms Steven Price
2024-06-10 13:41 ` [PATCH v3 32/43] arm_pmu: Provide a mechanism for disabling the physical IRQ Steven Price
2024-06-10 13:41 ` [PATCH v3 33/43] arm64: rme: Enable PMU support with a realm guest Steven Price
2024-06-10 13:41 ` [PATCH v3 34/43] kvm: rme: Hide KVM_CAP_READONLY_MEM for realm guests Steven Price
2024-06-10 13:41 ` [PATCH v3 35/43] arm64: RME: Propagate number of breakpoints and watchpoints to userspace Steven Price
2024-06-10 13:41 ` Steven Price [this message]
2024-06-10 13:41 ` [PATCH v3 37/43] arm64: RME: Initialize PMCR.N with number counter supported by RMM Steven Price
2024-06-10 13:41 ` [PATCH v3 38/43] arm64: RME: Propagate max SVE vector length from RMM Steven Price
2024-06-10 13:41 ` [PATCH v3 39/43] arm64: RME: Configure max SVE vector length for a Realm Steven Price
2024-06-10 13:41 ` [PATCH v3 40/43] arm64: RME: Provide register list for unfinalized RME RECs Steven Price
2024-06-10 13:42 ` [PATCH v3 41/43] arm64: RME: Provide accurate register list Steven Price
2024-06-10 13:42 ` [PATCH v3 42/43] arm64: kvm: Expose support for private memory Steven Price
2024-06-10 13:42 ` [PATCH v3 43/43] KVM: arm64: Allow activating realms Steven Price

Reply instructions:

You may reply publicly to this message via plain-text email
using any one of the following methods:

* Save the following mbox file, import it into your mail client,
  and reply-to-all from there: mbox

  Avoid top-posting and favor interleaved quoting:
  https://en.wikipedia.org/wiki/Posting_style#Interleaved_style

* Reply using the --to, --cc, and --in-reply-to
  switches of git-send-email(1):

  git send-email \
    --in-reply-to=20240610134202.54893-37-steven.price@arm.com \
    --to=steven.price@arm.com \
    --cc=alexandru.elisei@arm.com \
    --cc=catalin.marinas@arm.com \
    --cc=christoffer.dall@arm.com \
    --cc=gankulkarni@os.amperecomputing.com \
    --cc=james.morse@arm.com \
    --cc=jean-philippe@linaro.org \
    --cc=joey.gouly@arm.com \
    --cc=kvm@vger.kernel.org \
    --cc=kvmarm@lists.linux.dev \
    --cc=linux-arm-kernel@lists.infradead.org \
    --cc=linux-coco@lists.linux.dev \
    --cc=linux-kernel@vger.kernel.org \
    --cc=maz@kernel.org \
    --cc=oliver.upton@linux.dev \
    --cc=suzuki.poulose@arm.com \
    --cc=tabba@google.com \
    --cc=will@kernel.org \
    --cc=yuzenghui@huawei.com \
    /path/to/YOUR_REPLY

  https://kernel.org/pub/software/scm/git/docs/git-send-email.html

* If your mail client supports setting the In-Reply-To header
  via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox

all inboxes | Powered by JetHome®